// probably TODO: beep rotor 1 only every n rotations

// Board: NodeMCU 1.0 / (ESP-12-E Module)
// YUKIKAKI URI:  http://192.168.4.1

#define ROTOR1 D7
#define ROTOR2 D2
#define ROTOR3 D6
#define SPEAKER D5

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>

int     rpmLog1[720];         // rotations per minute of the last 12 hours for 3 rotors
int     rpmLog2[720];
int     rpmLog3[720];

int     rpmBuffer1[6];     // buffer stores rotations of 1 minute (10 seconds per slot)
int     rpmBuffer2[6];
int     rpmBuffer3[6];


boolean nRotor1 = 0;          // reed switch of rotors NOW
boolean nRotor2 = 0;
boolean nRotor3 = 0;

boolean pRotor1 = 0;          // reed switch of rotors in previous cycle
boolean pRotor2 = 0;
boolean pRotor3 = 0;

int     rpm1        = 0;      // current rpm of 3 rotors
int     rpm2        = 0;
int     rpm3        = 0;
int     maxRpm      = 100;    // maximum rpm of all rotors

String  ws          = "";     // html of website

unsigned long    nextMinute  = 0;     // next minute timer
unsigned long    nextSecond  = 0;     // next second timer
unsigned long    nextTenSeconds = 0;  // next ten seconds timer

boolean ledState    = 0;              // State of internal ESP LED

boolean debugMsg = true;

ESP8266WebServer server(80);
//====================================================== SETUP
void setup() {

  pinMode(LED_BUILTIN, OUTPUT);                       // initialize the ESP led pin as an output

  pinMode(ROTOR1,  INPUT_PULLUP);                     // rotor 1 top    - snowball tool
  pinMode(ROTOR2,  INPUT_PULLUP);                     // rotor 2 center - kids yukikaki
  pinMode(ROTOR3,  INPUT_PULLUP);                     // rotor 3 bottom - orange yukikaki
  pinMode(SPEAKER, OUTPUT);                           // speaker
  
  tone(SPEAKER, 250, 100);
  delay(500);

  Serial.begin(115200);                               // open serial connection with 115200bd
  if (debugMsg) Serial.println("--- YUKIKAKI RESEARCH STATION ---");
  if (debugMsg) Serial.println("t:" + String(millis()));

  // ................................................... open wifi access point
  if (debugMsg) Serial.println("Open wifi");
  WiFi.softAP("Yukikaki Research Station");
  server.on("/", handleRoot);
  server.begin();
}
//====================================================== LOOP
void loop() {

  delay(1);
  
  // ................................................... read rotor sensors
  nRotor1 = digitalRead(ROTOR1);
  nRotor2 = digitalRead(ROTOR2);
  nRotor3 = digitalRead(ROTOR3);

  // ................................................... check rotor1
  if (!nRotor1 && pRotor1) {                          // HL flange detected
    if (debugMsg) Serial.print("-Rotor1-");
    if (debugMsg) Serial.println(millis());
    rpmBuffer1[5]++;
    tone(SPEAKER, 1000, 50);
  }
  pRotor1 = nRotor1; // remember reed switch state

  // ................................................... check rotor2
  if (nRotor2 && !pRotor2) {                          // HL flange detected
    if (debugMsg) Serial.print("-Rotor2-");
    if (debugMsg) Serial.println(millis());
    rpmBuffer2[5]++;
    tone(SPEAKER, 750, 50);
  }
  pRotor2 = nRotor2; // remember reed switch state

  // ................................................... check rotor3
  if (!nRotor3 && pRotor3) {                          // HL flange detected
    if (debugMsg) Serial.print("-Rotor3-");
    if (debugMsg) Serial.println(millis());
    rpmBuffer3[5]++;
    tone(SPEAKER, 500, 50);
  }
  pRotor3 = nRotor3; // remember reed switch state

  // ................................................... do once every second:
  if (millis() > nextSecond) {
    nextSecond = millis() + 1000L;                    // update second timer
    digitalWrite(LED_BUILTIN, ledState = ! ledState); // blink internal ESP led
  }

  // ................................................... do once every ten seconds:
  if (millis() > nextTenSeconds) {
    nextTenSeconds = millis() + 10000L;               // update 10 seconds timer

    rpm1 = rpmBuffer1[0];                             // accumulate all rotations for the last minute (in 10-seconds segments)
    rpm2 = rpmBuffer2[0];
    rpm3 = rpmBuffer3[0];

    for (int i = 0; i < 5; i++) {
      rpmBuffer1[i] = rpmBuffer1[i + 1];              // move ring buffer
      rpmBuffer2[i] = rpmBuffer2[i + 1];
      rpmBuffer3[i] = rpmBuffer3[i + 1];

      rpm1 += rpmBuffer1[i];
      rpm2 += rpmBuffer2[i];
      rpm3 += rpmBuffer3[i];
    }
    
    rpmBuffer1[5] = 0;                                // reset newest 10-seconds slot
    rpmBuffer2[5] = 0;
    rpmBuffer3[5] = 0;

    rpm1/=2;                                          // some yukikakis always switch twice per rotation
    rpm2/=2;                                          
    createHtml();                                     // create new html
  }

  // ................................................... do once every minute:
  if (millis() > nextMinute) {
    nextMinute = millis() + 60000L;                   // update minute timer
    saveDataLog();                                    // save current rpm in log, calculate max rpm 
  }
  
  // ................................................... after 23 days of continuous operation:
  if (millis() > 2000000000L) {
    ESP.restart();                                    // soft reset
  }
  // ................................................... webserver
  server.handleClient();
}
